<?php
/**
 * <pre>
 * Invision Power Services
 * IP.Board v2.3.0
 * Basic Downloads Search
 * Last Updated: $Date: 2010-02-23 12:38:11 +0000 (Tue, 23 Feb 2010) $
 * </pre>
 *
 * @author 		$Author: matt $
 * @copyright	(c) 2001 - 2009 Invision Power Services, Inc.
 * @license		http://ipb.mxneo.ru/community/board/license.html
 * @package		IP.Downloads
 * @link		http://ipb.mxneo.ru
 * @version		$Rev: 5861 $
 *
 */

if ( ! defined( 'IN_IPB' ) )
{
	print "<h1>Incorrect access</h1>You cannot access this file directly. If you have recently upgraded, make sure you upgraded all the relevant files.";
	exit();
}

class search_engine_downloads extends search_engine
{
	/**
	 * Constructor
	 *
	 * @param	object	ipsRegistry reference
	 * @return	void
	 */
	public function __construct( ipsRegistry $registry )
	{
		/* Hard limit */
		IPSSearchRegistry::set('set.hardLimit', ( ipsRegistry::$settings['search_hardlimit'] ) ? ipsRegistry::$settings['search_hardlimit'] : 200 );

		parent::__construct( $registry );
	}
	
	/**
	 * Perform a search.
	 * Returns an array of a total count (total number of matches)
	 * and an array of IDs ( 0 => 1203, 1 => 928, 2 => 2938 ).. matching the required number based on pagination. The ids returned would be based on the filters and type of search
	 *
	 * So if we had 1000 replies, and we are on page 2 of 25 per page, we'd return 25 items offset by 25
	 *
	 * @return array
	 */
	public function search()
	{
		/* Init */
		$start				= intval( IPSSearchRegistry::get('in.start') );
		$perPage			= IPSSearchRegistry::get('opt.search_per_page');
		$sort_by			= IPSSearchRegistry::get('in.search_sort_by');
		$sort_order			= IPSSearchRegistry::get('in.search_sort_order');
		$search_term		= IPSSearchRegistry::get('in.clean_search_term');
		$content_title_only	= IPSSearchRegistry::get('opt.searchTitleOnly');
		$search_ids			= array();
		$groupby			= false;

		/* Sorting */
		switch( $sort_by )
		{
			default:
			case 'date':
				$sortKey	= 'file_submitted';
			break;
			case 'update':
				$sortKey	= 'file_updated';
			break;
			case 'title':
				$sortKey	= 'fordinal';
			break;
			case 'downloads':
				$sortKey	= 'file_downloads';
			break;
			case 'views':
				$sortKey	= 'file_views';
			break;
			case 'rating':
				$sortKey	= 'file_rating';
			break;
		}

		if ( $sort_order == 'asc' )
		{
			$this->sphinxClient->SetSortMode( SPH_SORT_ATTR_ASC, $sortKey );
		}
		else
		{
			$this->sphinxClient->SetSortMode( SPH_SORT_ATTR_DESC, $sortKey );
		}
			
		/* Limit Results */
		$this->sphinxClient->SetLimits( intval($start), intval($perPage) );

		/* Date limit */
		if ( $this->search_begin_timestamp )
		{
			if ( ! $this->search_end_timestamp )
			{
				$this->search_end_timestamp = time() + 100;
			}
			
			$this->sphinxClient->SetFilterRange( 'file_updated', $this->search_begin_timestamp, $this->search_end_timestamp );
		}
		
		/* Permissions/restrictions */
		if( $this->memberData['g_is_supmod'] )
		{
			$this->sphinxClient->SetFilter( 'file_open', array( 0, 1 ) );
		}
		else
		{
			$this->sphinxClient->SetFilter( 'file_open', array( 1 ) );
		}

		$this->_getDownloadsClasses();
		
		/* Generic category filtering */
		$_cats	= array( 0 );

		if( count( $this->registry->getClass('categories')->cat_lookup ) > 0 )
		{
			foreach( $this->registry->getClass('categories')->cat_lookup as $cid => $cinfo )
			{
				if( in_array( $cid, $this->registry->getClass('categories')->member_access['show'] ) )
				{
					$_cats[]	= $cid;
				}
			}
		}
		
		/* Did we search by category? - if so use those instead */
		if ( ! empty( ipsRegistry::$request['search_app_filters']['downloads']['downloads'] ) AND count( ipsRegistry::$request['search_app_filters']['downloads']['downloads'] ) )
		{
			/* Reset array */
			$_cats	= array( 0 );
			
			foreach(  ipsRegistry::$request['search_app_filters']['downloads']['downloads'] as $cat )
			{
				if( $cat )
				{
					if( in_array( $cid, $this->registry->getClass('categories')->member_access['show'] ) )
					{
						$_cats[]	= intval($cat);
					}
				}
			}
		}
		
		/* Filtering by paid or free? */
		if ( ! empty( ipsRegistry::$request['search_app_filters']['downloads']['freepaid'] ) )
		{
			switch( ipsRegistry::$request['search_app_filters']['downloads']['freepaid'] )
			{
				case 'free':
					$this->sphinxClient->SetFilterFloatRange( 'file_cost', "0", "0" );
				break;
				
				case 'paid':
					$this->sphinxClient->SetFilterFloatRange( 'file_cat', "0.01", "999999999.99" );
				break;
			}
		}

		$this->sphinxClient->SetFilter( 'file_cat', $_cats );

		/* Set search term */
		$_s		= '';
		
		if( $search_term )
		{
			if ( $content_title_only )
			{
				$_s		= '@file_name ' . $search_term;
			}
			else
			{
				$_s		= strstr( $search_term, '"' ) ? '@file_desc  ' . $search_term . ' | @file_name ' . $search_term : '@(file_desc,file_name) ' . $search_term;
			}
		}
		
		/* Run search and log warnings */
		$result	= $this->sphinxClient->Query( $_s, 'downloads_search_main,downloads_search_delta' );
		
		$this->logSphinxWarnings();
		
		if ( is_array( $result['matches'] ) && count( $result['matches'] ) )
		{
			foreach( $result['matches'] as $res )
			{
				$search_ids[] = intval($res['attrs']['search_id']);
			}
		}

		/* Return it */
		return array( 'count' => intval( $result['total_found'] ) > 1000 ? 1000 : $result['total_found'], 'resultSet' => $search_ids );
	}
	
	/**
	 * Perform the search.
	 * Returns an array with the results
	 *
	 * @param	array 		Member data for member we are searching
	 * @return	array
	 * @see		search_engine_downloads::search
	 */
	public function viewUserContent( $member )
	{
		/* Ensure we limit by date */
		$this->settings['search_ucontent_days'] = ( $this->settings['search_ucontent_days'] ) ? $this->settings['search_ucontent_days'] : 365;
	
		/* Set filters for search() method */
		IPSSearchRegistry::set('in.search_sort_by'   , 'date' );
		IPSSearchRegistry::set('in.search_sort_order', 'desc' );
		
		switch( IPSSearchRegistry::get('in.userMode') )
		{
			default:
			case 'all': 
				IPSSearchRegistry::set('opt.searchTitleOnly', false );
				IPSSearchRegistry::set('display.onlyTitles' , true );
				IPSSearchRegistry::set('opt.noPostPreview'  , false );
			break;
			case 'title': 
				IPSSearchRegistry::set('opt.searchTitleOnly', true );
				IPSSearchRegistry::set('display.onlyTitles' , true );
				IPSSearchRegistry::set('opt.noPostPreview'  , false );
			break;	
			case 'content': 
				IPSSearchRegistry::set('opt.searchTitleOnly', false );
				IPSSearchRegistry::set('display.onlyTitles' , false );
				IPSSearchRegistry::set('opt.noPostPreview'  , false );
			break;	
		}

		/* Filter by member? */
		$this->sphinxClient->SetFilter( 'file_submitter', array( intval( $member['member_id'] ) ) );
		
		/* Set timeframe cutoff */
		if ( $this->settings['search_ucontent_days'] )
		{
			$this->setDateRange( time() - ( 86400 * intval( $this->settings['search_ucontent_days'] ) ) );
		}
		
		/* Run search */
		return $this->search();
	}

	/**
	 * Perform the viewNewContent search.
	 * Returns an array with the results
	 *
	 * @return	array
	 * @see		search_engine_downloads::search
	 */
	public function viewNewContent()
	{
		$fids		= $this->registry->getClass('classItemMarking')->fetchCookieData( 'downloads', 'items' );
		$oldStamp	= $this->registry->getClass('classItemMarking')->fetchOldestUnreadTimestamp( array(), 'downloads' );
		$check		= IPS_UNIX_TIME_NOW - ( 86400 * 90 );
		
		IPSSearchRegistry::set('in.search_sort_by'   , 'date' );
		IPSSearchRegistry::set('in.search_sort_order', 'desc' );
		IPSSearchRegistry::set('opt.searchTitleOnly' , true );
		IPSSearchRegistry::set('display.onlyTitles'  , true );
		IPSSearchRegistry::set('opt.noPostPreview'   , true );
		
		if ( ! $this->memberData['bw_vnc_type'] )
		{
			$oldStamp	= IPSLib::fetchHighestNumber( array( intval( $this->memberData['_cache']['gb_mark__downloads'] ), intval( $this->memberData['last_visit'] ) ) );
			$fids		= array();
		}
		else
		{
			/**
			 * Check for additional read ids
			 */
			$this->_getDownloadsClasses();

			foreach( $this->registry->getClass('categories')->cat_lookup as $id => $data )
			{
				$lMarked	= $this->registry->getClass('classItemMarking')->fetchTimeLastMarked( array( 'forumID' => $id ), 'downloads' );
				
				if( $data['cfileinfo']['date'] > $lMarked )
				{
					$_fids = $this->registry->getClass('classItemMarking')->fetchReadIds( array( 'forumID' => $id ), 'downloads', false );
					
					if( is_array( $_fids ) )
					{
						foreach( $_fids as $k => $v )
						{
							$fids[ $k ]	= $v;
						}
					}
				}
			}

			if ( intval( $this->memberData['_cache']['gb_mark__downloads'] ) > 0  )
			{
				$oldStamp = $this->memberData['_cache']['gb_mark__downloads'];
			}
			
			/* Finalize times */
			if ( ! $oldStamp OR $oldStamp == IPS_UNIX_TIME_NOW )
			{
				$oldStamp = intval( $this->memberData['last_visit'] );
			}
			
			/* Older than 3 months.. then limit */
			if ( $oldStamp < $check )
			{
				$oldStamp = $check;
			}
		}
		
		/* Set the timestamps */
		$this->setDateRange( $oldStamp );

		/* Try and limit the files */
		if ( is_array( $fids ) AND count( $fids ) )
		{
			if ( count( $fids ) > 300 )
			{
				/* Sort by last read date */
				arsort( $fids, SORT_NUMERIC );
				$fids = array_slice( $fids, 0, 300 );
			}
							
			$this->DB->build( array( 'select' => 'file_id, file_updated',
							   		 'from'   => 'downloads_files',
							   	     'where'  => "file_id IN (" . implode( ",", array_keys( $fids ) ) . ')' ) );
							   
			$this->DB->execute();
			
			while( $row = $this->DB->fetch() )
			{
				/* Posted in since last read? */
				if ( $row['file_updated'] > $fids[ $row['file_id'] ] )
				{
					unset( $fids[ $row['file_id'] ] );
				}
			}
			
			if ( count( $fids ) )
			{
				$fids = array_keys( $fids );
			}
		}
		
		/* Set read files */
		if ( count( $fids ) )
		{
			$this->sphinxClient->SetFilter( 'search_id', $fids, TRUE );
		}
		
		/* Set up some vars */
		IPSSearchRegistry::set('set.resultCutToDate', $oldStamp );
		
		return $this->search();
	}
	
	/**
	 * Perform the search.
	 * Returns an array with the results
	 *
	 * @return	array
	 * @see		search_engine_downloads::search
	 */
	public function viewActiveContent()
	{
		$seconds = IPSSearchRegistry::get('in.period_in_seconds');

		IPSSearchRegistry::set('in.search_sort_by'   , 'date' );
		IPSSearchRegistry::set('in.search_sort_order', 'desc' );
		IPSSearchRegistry::set('opt.searchTitleOnly' , true );
		IPSSearchRegistry::set('display.onlyTitles'  , true );
		IPSSearchRegistry::set('opt.noPostPreview'   , true );

		/* Set the timestamps */
		$this->setDateRange( intval( time() - $seconds ) );
		
		return $this->search();
	}
	
	/**
	 * Remap standard columns (Apps can override )
	 *
	 * @param	string	$column		sql table column for this condition
	 * @return	string				column
	 * @return	void
	 */
	public function remapColumn( $column )
	{
		$column = $column == 'member_id' ? 'f.file_submitter' : $column;

		return $column;
	}
	
	/**
	 * Get our helper classes
	 *
	 * @return void
	 */
	protected function _getDownloadsClasses()
	{
		if( !$this->registry->isClassLoaded('categories') )
		{
			$classToLoad			= IPSLib::loadLibrary( IPSLib::getAppDir( 'downloads' ) . '/app_class_downloads.php', 'app_class_downloads', 'downloads' );
			$app_class_downloads	= new $classToLoad( $this->registry );
		}
	}

	/**
	 * Returns an array used in the searchplugin's setCondition method
	 *
	 * @param	array 	$data	Array of forums to view
	 * @return	array 	Array with column, operator, and value keys, for use in the setCondition call
	 */
	public function buildFilterSQL( $data )
	{
		/* INIT */
		$return = array();
		
		/* Set up some defaults */
		IPSSearchRegistry::set( 'opt.noPostPreview'  , false );
		IPSSearchRegistry::set( 'opt.onlySearchPosts', false );
		
		return $return;
	}
}